@@ -1,8 +1,9 @@
 /**
  * @file notify.c Notification API
  * @ingroup core
- *
- * purple
+ */
+
+/* purple
  *
  * Purple is the legal property of its developers, whose names are too numerous
  * to list here.  Please refer to the COPYRIGHT file distributed with this
@@ -20,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include "internal.h"
 #include "dbus-maybe.h"
@@ -112,6 +113,10 @@ purple_notify_email(void *handle, const char *subject, const char *from,
 		info            = g_new0(PurpleNotifyInfo, 1);
 		info->type      = PURPLE_NOTIFY_EMAIL;
 		info->handle    = handle;
+
+		purple_signal_emit(purple_notify_get_handle(), "displaying-email-notification",
+							subject, from, to, url);
+
 		info->ui_handle = ops->notify_email(handle, subject, from, to, url);
 		info->cb = cb;
 		info->cb_user_data = user_data;
@@ -145,8 +150,6 @@ purple_notify_emails(void *handle, size_t count, gboolean detailed,
 {
 	PurpleNotifyUiOps *ops;
 
-	g_return_val_if_fail(count != 0, NULL);
-
 	if (count == 1) {
 		return purple_notify_email(handle,
 								 (subjects == NULL ? NULL : *subjects),
@@ -164,6 +167,10 @@ purple_notify_emails(void *handle, size_t count, gboolean detailed,
 		info            = g_new0(PurpleNotifyInfo, 1);
 		info->type      = PURPLE_NOTIFY_EMAILS;
 		info->handle    = handle;
+
+		purple_signal_emit(purple_notify_get_handle(), "displaying-emails-notification",
+							subjects, froms, tos, urls, count);
+
 		info->ui_handle = ops->notify_emails(handle, count, detailed, subjects,
 											 froms, tos, urls);
 		info->cb = cb;
@@ -497,7 +504,7 @@ purple_notify_user_info_entry_destroy(PurpleNotifyUserInfoEntry *user_info_entry
 	g_return_if_fail(user_info_entry != NULL);
 	
 	g_free(user_info_entry->label);
-	g_free(user_info_entry->value);	
+	g_free(user_info_entry->value);
 	PURPLE_DBUS_UNREGISTER_POINTER(user_info_entry);
 	g_free(user_info_entry);
 }
@@ -559,7 +566,7 @@ purple_notify_user_info_get_text_with_newline(PurpleNotifyUserInfo *user_info, c
 		if (user_info_entry->label && user_info_entry->value)
 			g_string_append(text, ": ");
 		if (user_info_entry->value)
-			g_string_append(text, user_info_entry->value);			
+			g_string_append(text, user_info_entry->value);
 
 		/* Display a section break as a horizontal line */
 		if (user_info_entry->type == PURPLE_NOTIFY_USER_INFO_ENTRY_SECTION_BREAK)
@@ -681,8 +688,11 @@ purple_notify_user_info_add_section_break(PurpleNotifyUserInfo *user_info)
 void
 purple_notify_user_info_remove_last_item(PurpleNotifyUserInfo *user_info)
 {
-	user_info->user_info_entries = g_list_remove(user_info->user_info_entries,
-												 g_list_last(user_info->user_info_entries)->data);
+	GList *last = g_list_last(user_info->user_info_entries);
+	if (last) {
+		purple_notify_user_info_entry_destroy(last->data);
+		user_info->user_info_entries = g_list_delete_link(user_info->user_info_entries, last);
+	}
 }
 
 void *
@@ -799,6 +809,21 @@ purple_notify_init(void)
 {
 	gpointer handle = purple_notify_get_handle();
 
+	purple_signal_register(handle, "displaying-email-notification",
+						 purple_marshal_VOID__POINTER_POINTER_POINTER_POINTER, NULL, 4,
+						 purple_value_new(PURPLE_TYPE_STRING),
+						 purple_value_new(PURPLE_TYPE_STRING),
+						 purple_value_new(PURPLE_TYPE_STRING),
+						 purple_value_new(PURPLE_TYPE_STRING));
+
+	purple_signal_register(handle, "displaying-emails-notification",
+						 purple_marshal_VOID__POINTER_POINTER_POINTER_POINTER_UINT, NULL, 5,
+						 purple_value_new(PURPLE_TYPE_POINTER),
+						 purple_value_new(PURPLE_TYPE_POINTER),
+						 purple_value_new(PURPLE_TYPE_POINTER),
+						 purple_value_new(PURPLE_TYPE_POINTER),
+						 purple_value_new(PURPLE_TYPE_UINT));
+
 	purple_signal_register(handle, "displaying-userinfo",
 						 purple_marshal_VOID__POINTER_POINTER_POINTER, NULL, 3,
 						 purple_value_new(PURPLE_TYPE_SUBTYPE,
