@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  */
 
@@ -45,7 +45,7 @@ typedef enum profile_lang_id {
 	ES_AR, ES_ES, ES_MX, ES_US,
 	FR_CA, FR_FR, 
 	IT, JA, KO, NO, PT, SV, 
-	ZH_CN, ZH_HK, ZH_TW, ZH_US
+	ZH_CN, ZH_HK, ZH_TW, ZH_US, PT_BR
 } profile_lang_id_t;
 
 typedef struct profile_lang_node {
@@ -124,6 +124,7 @@ static const profile_lang_node_t profile_langs[] = {
 	{ KO,    "\xb0\xbb\xbd\xc5\x20\xb3\xaf\xc2\xa5&nbsp;",           NULL },
 	{ NO,    "Sist oppdatert&nbsp;",                                 NULL },
 	{ PT,    "\332ltima atualiza\347\343o&nbsp;",                    NULL },
+	{ PT_BR, "\332ltima atualiza\347\343o:",                         NULL },
 	{ SV,    "Senast uppdaterad&nbsp;",                              NULL },
 	{ ZH_CN, "\xd7\xee\xba\xf3\xd0\xde\xb8\xc4\xc8\xd5\xc6\xda",     NULL },
 	{ ZH_HK, "\xb3\xcc\xaa\xf1\xa7\xf3\xb7\x73\xae\xc9\xb6\xa1",     NULL },
@@ -506,6 +507,29 @@ static const profile_strings_node_t profile_strings[] = {
 		"Site legal 3:",
 		NULL
 	},
+	{ PT_BR, "pt_br", "ISO-8859-1",
+		"ID Yahoo!:",
+		"Particular",
+		"Sem resposta",
+		"Meu e-mail",
+		"Nome verdadeiro:",
+		"Localização:",
+		"Idade:",
+		"Estado civil:",
+		"Sexo:",
+		"Ocupação:",
+		"Pasatiempos:",
+		"Últimas novidades:",
+		"Frase preferida:",
+		"Links",
+		"Nenhuma home page especificada",
+		"Página Web:",
+		"Nenhum site legal especificado",
+		"Link legal 1",
+		"Link legal 2",
+		"Link legal 3",
+		NULL
+	},
 	{ SV, "sv", "ISO-8859-1",
 		"Yahoo!-ID:",
 		"Privat",
@@ -908,11 +932,17 @@ static void yahoo_got_info(PurpleUtilFetchUrlData *url_data, gpointer user_data,
 	/* Try to put the photo in there too, if there's one */
 	if (photo_url_text) {
 		PurpleUtilFetchUrlData *url_data;
+		gboolean use_whole_url = FALSE;
+
+		/* use whole URL if using HTTP Proxy */
+		if ((info_data->gc->account->proxy_info) && (info_data->gc->account->proxy_info->type == PURPLE_PROXY_HTTP))
+		    use_whole_url = TRUE;
+
 		/* User-uploaded photos use a different server that requires the Host
 		 * header, but Yahoo Japan will use the "chunked" content encoding if
 		 * we specify HTTP 1.1. So we have to specify 1.0 & fix purple_util_fetch_url
 		 */
-		url_data = purple_util_fetch_url(photo_url_text, FALSE, NULL,
+		url_data = purple_util_fetch_url(photo_url_text, use_whole_url, NULL,
 				FALSE, yahoo_got_photo, info2_data);
 		if (url_data != NULL)
 			yd->url_datas = g_slist_prepend(yd->url_datas, url_data);
@@ -1021,7 +1051,8 @@ yahoo_got_photo(PurpleUtilFetchUrlData *url_data, gpointer data,
 			purple_debug_info("yahoo", "Error getting %s: %s\n",
 					photo_url_text, url_text);
 		} else {
-			purple_debug_info("yahoo", "%s is %d bytes\n", photo_url_text, len);
+			purple_debug_info("yahoo", "%s is %" G_GSIZE_FORMAT
+					" bytes\n", photo_url_text, len);
 			id = purple_imgstore_add_with_id(g_memdup(url_text, len), len, NULL);
 			
 			tmp = g_strdup_printf("<img id=\"%d\"><br>", id);
